@@ -1,7 +1,7 @@
 from app import app
 from flask.ext.appbuilder.security.models import is_menu_public, is_item_public
 from flask.ext.appbuilder.models.datamodel import SQLAModel
-from flask import g, request
+from flask import g, request, url_for
 from flask.ext.login import current_user
 
 
@@ -24,48 +24,57 @@ class TemplateFilters(object):
                 attr = getattr(self, attr_name)
                 app.jinja_env.filters[attr._filter] = attr
 
+
     @app_template_filter('link_order')
-    def link_order_filter(self, s):
-
-        if (request.args.get('order_column') == s):
-            lststr = request.path.split('?')
-            if (request.args.get('order_direction') == 'asc'):
-                return lststr[0] + '?order_column=' + s + '&order_direction=desc'
+    def link_order_filter(self, column, generalview_name):
+        """
+        Arguments are passed like: _oc_<VIEW_NAME>=<COL_NAME>&_od_<VIEW_NAME>='asc'|'desc'
+        """
+        new_args = request.view_args.copy()
+        args = request.args.copy()
+        if ('_oc_' + generalview_name) in args:
+            args['_oc_' + generalview_name] = column
+            if args.get('_od_' + generalview_name) == 'asc':
+                args['_od_' + generalview_name] = 'desc'
             else:
-                return  request.path + '?order_column=' + s + '&order_direction=asc'
+                args['_od_' + generalview_name] = 'asc'
         else:
-            return  request.path + '?order_column=' + s + '&order_direction=asc'
+            args['_oc_' + generalview_name] = column
+            args['_od_' + generalview_name] = 'asc'
+        return url_for(request.endpoint,**dict(new_args.items() + args.to_dict().items()))
 
 
+    @app_template_filter('link_page')
+    def link_page_filter(self, page, generalview_name):
+        """
+        Arguments are passed like: page_<VIEW_NAME>=<PAGE_NUMBER>
+        """
+        new_args = request.view_args.copy()
+        args = request.args.copy()
+        args['page_' + generalview_name] = page
+        return url_for(request.endpoint,**dict(new_args.items() + args.to_dict().items()))
+                
 
     @app_template_filter('get_link_next')
     def get_link_next_filter(self, s):
         return request.args.get('next')
 
-
+    # to improve
     @app_template_filter('set_link_filters')
-    def add_link_filters_filter(self, path, filters):
+    def set_link_filters_filter(self, path, filters, pk):
         lnkstr = path
+        
         for _filter in filters:
-            try:
-                datamodel = SQLAModel(filters.get(_filter))
-                pk_name = datamodel.get_pk_name()
-                pk_value = getattr(filters.get(_filter), pk_name)
-                lnkstr = lnkstr + '&_flt_' + _filter + '=' + str(pk_value)
-            except:
-                pass
-        return lnkstr
-
+            lnkstr = lnkstr + '&_flt_' + _filter + '=' + str(pk)
+        return lnkstr        
 
     @app_template_filter('get_link_order')
-    def get_link_order_filter(self, s):
-        if request.args.get('order_column') == s:
-            order_direction = request.args.get('order_direction')
-            if (order_direction):
-                if (order_direction == 'asc'):
-                    return 2
-                else:
-                    return 1
+    def get_link_order_filter(self, column, generalview_name):
+        if request.args.get('_oc_' + generalview_name) == column:
+            if (request.args.get('_od_' + generalview_name) == 'asc'):
+                return 2
+            else:
+                return 1
         else:
             return 0
 
