@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-
+from itertools import groupby
 from flask import flash
 from flask.ext.babel import gettext, ngettext, lazy_gettext
 from sqlalchemy.orm import class_mapper, joinedload
@@ -52,22 +52,44 @@ class SQLAModel(DataModel):
                 else:
                     pass
             except:
-                if isinstance(self.obj.__mapper__.columns[filter_key].type, sa.types.String):
-                    query = query.filter(getattr(self.obj,filter_key).like(filters.get(filter_key) + '%%'))
+                if self.is_string(filter_key) or self.is_text(filter_key):
+                    query = query.filter(getattr(self.obj,filter_key).like(filters.get(filter_key) + '%'))
         if (order_column != ''):
             query = query.order_by(order_column + ' ' + order_direction)
+        
         return query
-    
+
+   
     """
     QUERY
     filters: dict with filters {<col_name>:<value,...}
     order_column
     order_direction: <'asc'|'desc'>
     """
-    def query(self, filters = {}, order_column = '', order_direction = ''):
-        query = self.obj.query
-        query = self._get_base_query(query = query, filters = filters, order_column = order_column, order_direction = order_direction)
-        return query.all()
+    def query(self, filters = {}, order_column = '', order_direction = '',
+                page = None, page_size = None):
+        query = self.session.query(self.obj)
+        query_count = self.session.query(func.count('*')).select_from(self.obj)
+        
+        query_count = self._get_base_query(query = query_count, 
+                        filters = filters, 
+                        order_column = order_column, 
+                        order_direction = order_direction)
+        
+        query = self._get_base_query(query = query, 
+                        filters = filters, 
+                        order_column = order_column, 
+                        order_direction = order_direction)
+        
+        count = query_count.scalar() 
+        
+        if page: 
+            print "OFFSET", page*page_size
+            query = query.offset(page*page_size)
+        if page_size:
+            query = query.limit(page_size)
+        
+        return count, query.all()
 
 
     def query_simple_group(self, group_by = '', filters = {}, order_column = '', order_direction = ''):
@@ -79,11 +101,31 @@ class SQLAModel(DataModel):
             query = query.group_by(rel_model)
             return query.all()
         except:
-            query = self.session.query(rel_model,func.count(self.obj.id))
+            query = self.session.query(group_by,func.count(self.obj.id))
             query = self._get_base_query(query = query, filters = filters, order_column = order_column, order_direction = order_direction)
             query = query.group_by(group_by)
             return query.all()
 
+    def grouper(self, item):
+        pass
+
+    def query_month_group(self, group_by = '', filters = {}, order_column = '', order_direction = ''):
+        query = self.session.query(self.obj)
+        query = self._get_base_query(query = query, filters = filters, order_column = group_by, order_direction = 'asc')
+        query_result = query.all()
+        retlst = []
+        for ( grouped, items ) in groupby( query_result, lambda x: (getattr(x,group_by).month,getattr(x,group_by).year) ):
+            retlst.append([grouped, len(list(items))])
+        return retlst
+        
+    def query_year_group(self, group_by = '', filters = {}, order_column = '', order_direction = ''):
+        query = self.session.query(self.obj)
+        query = self._get_base_query(query = query, filters = filters, order_column = group_by, order_direction = 'asc')
+        query_result = query.all()
+        retlst = []
+        for ( grouped, items ) in groupby( query_result, lambda x: (getattr(x,group_by).year)):
+            retlst.append([grouped, len(list(items))])
+        return retlst
 
     """
     -----------------------------------------
